from flask import Flask, render_template, jsonify, request
from copy import copy
import uuid
app = Flask(__name__)

# Vitals
vitalsHeartrate = -1
vitalsOxygen = -1
vitalsBodytemp = -1.0

@app.route('/api/vitals/heartrate')
def getVitalsHeartrate():
	returnArr = [ { 'heartrate': vitalsHeartrate } ]
	return jsonify(returnArr)

@app.route('/api/vitals/heartrate', methods=['POST'])
def setVitalsHeartrate():
	global vitalsHeartrate
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals/heartrate'):
			return 'Forbidden.', 403

		vitalsHeartrate = json['heartrate']
	except:
		return 'Incorrect usage.\nUsage: { heartrate: INT, uuid: STRING }\n', 400
	return 'Information set successfully', 204

@app.route('/api/vitals/oxygen')
def getVitalsOxygen():
	returnArr = [ { 'oxygen': vitalsOxygen } ]
	return jsonify(returnArr)

@app.route('/api/vitals/oxygen', methods=['POST'])
def setVitalsOxygen():
	global vitalsOxygen
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals/oxygen'):
			return 'Forbidden.', 403

		vitalsOxygen = json['oxygen']
	except:
		return 'Incorrect usage.\nUsage: { oxygen: INT }\n', 400
	return 'Information set successfully', 204

@app.route('/api/vitals/bodytemp')
def getVitalsBodytemp():
	returnArr = [ { 'bodytemp': vitalsBodytemp } ]
	return jsonify(returnArr)

@app.route('/api/vitals/bodytemp', methods=['POST'])
def setVitalsBodytemp():
	global vitalsBodytemp
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals/bodytemp'):
			return 'Forbidden.', 403

		vitalsBodytemp = json['bodytemp']
	except:
		return 'Incorrect usage.\nUsage: { bodytemp: FLOAT }\n', 400
	return 'Information set successfully', 204

@app.route('/api/vitals')
def getVitals():
	returnArr = [ { 'heartrate': vitalsHeartrate, 'oxygen': vitalsOxygen, 'bodytemp': vitalsBodytemp } ]
	return jsonify(returnArr)

@app.route('/api/vitals', methods=['POST'])
def setVitals():
	global vitalsHeartrate
	global vitalsOxygen
	global vitalsBodytemp

	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals'):
			return 'Forbidden.', 403
		# This is a bit ugly but its just how I'm checking that everything is there without setting variables if the json is incorrect
		tempH = json['heartrate'] 
		tempO = json['oxygen']
		tempB = json['bodytemp']
		vitalsHeartrate = tempH
		vitalsOxygen = tempO
		vitalsBodytemp = tempB
	except:
		return 'Incorrect usage.\nUsage: { heartrate: INT, oxygen: INT, bodytemp: FLOAT }\n', 400
	return 'Information set successfully', 204

# Fitness
fitnessSteps = -1

@app.route('/api/fitness/steps')
def getSteps():
	return getFitness() # This is the same for now.

@app.route('/api/fitness/steps', methods=['POST'])
def setFitnessSteps():
	global fitnessSteps
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/fitness/steps'):
			return 'Forbidden.', 403

		vitalsBodytemp = json['steps']
	except:
		return 'Incorrect usage.\nUsage: { steps: INT }\n', 400
	return 'Information set successfully', 204


@app.route('/api/fitness')
def getFitness():
	returnArr = [ { 'steps': fitnessSteps } ]
	return jsonify(returnArr)

@app.route('/api/fitness', methods=['POST'])
def setFitness():
	global fitnessSteps
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/fitness'):
			return 'Forbidden.', 403

		fitnessSteps = json['steps']
	except:
		return 'Incorrect usage.\nUsage: { steps: INT }\n', 400
	return 'Information set successfully', 204


# Cyberware management
cyberware = []
newCyberwareTemplate = { "uuid": -1, "name": "", "hotpluggable": False, "lastMalfunction": "", "frontendAwknowledge": False, "canSet": [] }


# This makes the system aware of a new piece of hardware.
# While, for the most part, not required due to the design of this project,
#  it's handy for error reporting and communication between hardware and
#  the end user.
#
# Arguments: { name: STRING, hotpluggable: BOOL, canSet: ARRAY }
# Returns:   { uuid: INT }
# name: A human-readable name
# hotpluggable argument: Used for removable modules known as shards.
#
# uuid: Returns a uuid for the hardware.
@app.route('/api/cyberware/add', methods=['POST'])
def addCyberware():
	global cyberware
	json = request.get_json()

	try:
		tempName = json['name']
		tempHotpluggable = json['hotpluggable']
		tempNewCyberware = copy(newCyberwareTemplate)

		tempNewCyberware['uuid'] = str(uuid.uuid4())
		tempNewCyberware['name'] = tempName
		tempNewCyberware['hotpluggable'] = tempHotpluggable

		cyberware.append(tempNewCyberware)
	except:
		return 'Incorrect usage.\nUsage: { name: STRING, hotpluggable: BOOL, canSet: ARRAY }\n', 400
	return jsonify([ { "uuid": tempNewCyberware['uuid'] } ]), 204

# Arguments: { uuid: INT }
@app.route('/api/cyberware/remove', methods=['POST'])
def removeCyberware():
	global cyberware
	json = request.get_json()

	try:
		desiredId = json['uuid']
		i = 0
		for c in cyberware:
			if c['uuid'] == desiredId:
				del cyberware[i]
				return 'Cyberware removed', 204
			i = i + 1
	except:
		return 'Incorrect usage.\nUsage: { uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

# Returns: { uuid: STRING, name: STRING, hotpluggable: BOOL, lastMalfunction: STRING, frontendAwknowledge: BOOL }
# uuid:               Unique identifier of the hardware
# name:               Human-readable name
# hotpluggable:       Hardware can be removed during runtime.
# lastMalfunction:    A string with information on the last malfunction.
# frontendAwknowlege: A tool for the frontend to keep track of whether it's aware of the hardware.
# TODO: If this method is to exist, it needs to return everything WITHOUT UUID's.
@app.route('/api/cyberware')
def getCyberware():
	return 'Not implemented', 501

	#if not authenticate(json['uuid'], '/api/cyberware'):
	#		return 'Forbidden.', 403
	r#eturn jsonify(cyberware)

@app.route('/api/cyberware/get')
def getCyberwareSpecific():
	try:
		desiredId = json['uuid']
		i = 0
		for c in cyberware:
			if c['uuid'] == desiredId:
				return jsonify(c), 204
				break;
	except:
		return 'Incorrect usage.\nUsage: { uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

# Arguments { uuid: INT }
@app.route('/api/cyberware/reset', methods=['POST'])
def resetCyberwareMalfunction():
	#if not authenticate(json['uuid'], '/api/cyberware/reset'):
	#		return 'Forbidden.', 403

	return 'Not Implemented', 501

def authenticate(uuid, endpoint):
	for c in cyberware: # UUID Match
		if c['uuid'] == uuid:
			requestedHardware = c
			break

	for e in requestedHardware['canSet']: # Endpoint match
		if e == endpoint:
			return True
	return False
		

@app.route('/')
def uiindex():
	return render_template('index.html')
